#import "Controller.h"
#include <QuickTime/QuickTime.h>
#include "QTUtils.h"
#import "ActiveDeveloperLib.h"

static void * gControllerObject;

@implementation Controller
- (NSApplicationTerminateReply)applicationShouldTerminate:(NSApplication *)sender
{
	ActiveDeveloper_applicationShouldTerminate(sender);
	return YES;
}

- (void)applicationDidFinishLaunching:(NSNotification *)notification;	
{
	if (ActiveDeveloper_connectToIDE())
		NSLog(@" Application entered an ActiveDeveloper session ... ");
	else	NSLog(@" Application running StandAlone from ActiveDeveloper ...");
}

//
- (IBAction)goToBeginning:(id)sender
{						// Called when the go to beginning button is pressed
    [self saveCurrentMoviePlayingState];	// save current movie state
    [movieViewObject gotoBeginning:sender];	// go to the beginning of the movie   	
    [self restoreMoviePlayingState:sender];	// restore movie state
}

- (IBAction)goToEnd:(id)sender
{					// Called when the go to beginning button is pressed
    [movieViewObject gotoEnd:sender];	// go to the end of the movie
}


- (IBAction)play:(id)sender
{							//  Called when the play button is pressed.
    // has the movie just finished playing?
    if (IsMovieDone(gMovie)) {
        [movieViewObject gotoBeginning:sender];		// yes, so go to the beginning
    }
    
    if ([movieViewObject isPlaying]) {			// is the movie currently playing?
        [movieViewObject stop:sender];			// yes, so stop it
    }
    else						// movie is not currently playing
    {
        [movieViewObject start:sender];			// go ahead and start it playing
    }
    
    // change the button text to "pause"
    [sender setTitle:@"||"];
    
    // set button action method so next time it is invoked, our "stop" method will instead be called
    [sender setAction:@selector(stop:)];
}

- (IBAction)stop:(id)sender
{							// Called when the stop button is pressed.
    [movieViewObject stop:sender];			// stop the movie
    [self resetPlayButtonForMovieStopState:sender];	// adjust the play button state
}

- (IBAction)setVolume:(id)sender
{	// Called when the volume slider is pressed.
    // set the movie volume to correspond to the current value of the slider
    SetMovieVolume([[movieViewObject movie] QTMovie],[setVolumeSlider floatValue]);
}

- (IBAction)stepBack:(id)sender
{							// Called when the step-back button is pressed.
    [self saveCurrentMoviePlayingState];		// save the current movie play state
    [movieViewObject stepBack:sender];			// step the movie back
    [self restoreMoviePlayingState:sender];		// restore the movie play state
}

- (IBAction)stepForward:(id)sender
{							// Called when the step-forward button is pressed.
    // has the movie just finished playing?
    if (IsMovieDone(gMovie))
    {
        GoToBeginningOfMovie(gMovie);			// yes, so go to beginning
    }

    [self saveCurrentMoviePlayingState];		// save the current play state
    [movieViewObject stepForward:sender];		// step the movie forward
    [self restoreMoviePlayingState:sender];		// restore the movie play state
}

- (void)resetPlayButtonForMovieStopState:(id)sender
{
    // reset button text to "play" text
    [playButton setTitle: @">"];
    // reset button action method to our "play" method so the next time the button is pressed 
    // this method will instead be invoked
    [playButton setAction: @selector(play:)];

}

- (void)showTheWindow:(NSWindow *)window
{						// make a window visible and bring it to the front
    // is the window visible?
    if (![window isVisible])
    {
        // window is not visible, so let's make it visible
        [window setFrame:[window frame] display:YES];
    }
    // make the window accept keys, and make it the front window
    [window makeKeyAndOrderFront:menuItem_New];

}

- (IBAction)movieProperties:(id)sender
{   // Called when the Properties menu item in the Movie menu is selected.
    // display the movie properties window
    [self showTheWindow:moviePropertiesWindow];
}

- (IBAction)newWindow:(id)sender
{	// Called when the "New" menu item in the file menu is selected.
    [self showTheWindow:movieWindow];
}

//
- (void)awakeFromNib
{	// Called during NIB loading after all our objects are unarchived and connected
 NSString *filePath = [[NSBundle mainBundle] pathForResource:@"MovieFile" ofType:@"mov"];
 NSURL *movieUrl = [NSURL fileURLWithPath:filePath];
 NSMovie *theMovie = [[NSMovie alloc] initWithURL:movieUrl byReference:YES];
 Movie qtmovie = [theMovie QTMovie];	 // retrieve QuickTime-style movie (type "Movie" from QuickTime/Movies.h)
    
    // save (for later use) the QuickTime-style movie type for our NSMovie
    gMovie = qtmovie;

    // assign the NSMovie object (which we made above) for our NSMovieView object
    [movieViewObject setMovie: theMovie];

    // set the high/low ranges for our volume slider
    [setVolumeSlider setMaxValue: 256];
    [setVolumeSlider setMinValue: 0];

    [menuItem_New setEnabled:NSOffState];
    
    // we must initialize the movie toolbox before calling any of it's functions
    EnterMovies();

    // save a copy of the Controller object so we can later call methods
    //    in the class from _outside_ the actual class implementation
    saveObjectForCallback(self);
    
    // setup a callback for our movie so QuickTime will call us when the movie finishes playing
    gMoviePlayingCompleteCallBack =  NewQTCallBackUPP(&MyCallBackProc);
	[self setupMoviePlayingCompleteCallback:qtmovie callbackUPP:gMoviePlayingCompleteCallBack];
    
    // set the volume slider to the current volume of our movie */
    [setVolumeSlider setFloatValue: GetMovieVolume(qtmovie)];
    
    // the movie properties window will display miscellaneous properties for the movie 
    // here we do the setup for all the controls in this window
    [self setMoviePropertyWindowControlValues:qtmovie];
    
}

- (void)setMoviePropertyWindowControlValues:(Movie)qtmovie
{	// The movie properties window displays miscellaneous properties for the movie.
    if (QTUtils_IsAutoPlayMovie (qtmovie))
    {
        [autoPlayEnabled setStringValue:@"Yes"];
    }
    else
    {
        [autoPlayEnabled setStringValue:@"No"];
    }

    [trackCount setIntValue:GetMovieTrackCount(qtmovie)];

    [self buildTrackMediaTypesArray:qtmovie];
        
    [movieTimeScale setStringValue:GetMovieTimeScaleAsString(qtmovie)];
    [movieDuration setStringValue:GetMovieDurationAsString(qtmovie)];
}

//
- (id)tableView:(NSTableView *)aTableView
    objectValueForTableColumn:(NSTableColumn *)aTableColumn
    row:(int)rowIndex
{						// Movie Window TableView callback
    // turn editing off for each cell
    [aTableColumn setEditable:NO];
    
    // populate the list with data from our movie track media types array
    return ([movieTrackMediaTypesArray objectAtIndex:rowIndex]);
}

- (int)numberOfRowsInTableView:(NSTableView *)aTableView
{
    return [movieTrackMediaTypesArray count];	// Movie Window TableView callback
}

//
- (id)init
{
	// allocate an array to store the track media types for our movie 
	// these will be displayed in our NSTableView control in our movie properties window
    movieTrackMediaTypesArray=[[NSMutableArray alloc] init];
    
    return self;
}

-(void)dealloc
{
    [movieTrackMediaTypesArray release];
    DisposeCallBack(gQtCallBack);
}

- (void)buildTrackMediaTypesArray:(Movie)qtmovie
{		// Build an array of track media types for our movie. to display in Movie Window
 short i;
    
    for (i = 0; i < GetMovieTrackCount(qtmovie); ++i)
    {
	Str255			mediaName;
        OSErr			myErr;
        Track			movieTrack = GetMovieIndTrack(qtmovie, i+1); // tracks start at index 1
        Media			trackMedia = GetTrackMedia(movieTrack);
        MediaHandler	trackMediaHandler = GetMediaHandler(trackMedia);
		
        // get the text of the media type
	myErr = MediaGetName(trackMediaHandler, mediaName, 0, NULL);

        // add the media type string to our array (as an NSString)
        [movieTrackMediaTypesArray insertObject:[NSString stringWithCString:&mediaName[1] length:mediaName[0]] atIndex:i];
    }

}

- (void)saveCurrentMoviePlayingState
{
    gIsMoviePlaying = [movieViewObject isPlaying];	// Save the current movie playing state.
    gMoviePlaybackRate = [movieViewObject rate];
}

- (void)restoreMoviePlayingState:(id)sender
{
    if (gIsMoviePlaying)	[movieViewObject start:sender];	// Restore the current movie playing state.
    
    [movieViewObject setRate:gMoviePlaybackRate];
}

- (void)restoreMoviePlayCompleteCallBack
{	// reinstall QuickTime callback
    [self setupMoviePlayingCompleteCallback:gMovie callbackUPP:gMoviePlayingCompleteCallBack];
}

- (void)setupMoviePlayingCompleteCallback:(Movie)theMovie callbackUPP:(QTCallBackUPP) callbackUPP
{	//  Here we specify QuickTime call us when the movie stops playing.
    TimeBase tb = GetMovieTimeBase (theMovie);
    OSErr err = noErr;
    
    gQtCallBack = NewCallBack (tb, callBackAtExtremes);
    if (gQtCallBack != NULL)
    {
        err = CallMeWhen (gQtCallBack, 
                        callbackUPP, 
                        NULL,			// refCon
                        triggerAtStop, 		// flags - call us when stopped
                        NULL, 			// param2 - don't care
                        NULL);			// param3 - don't care
    }
}

@end


void adjustPlayButtonForMyMovieControllerObject()
{	// Once the movie has stopped playing, QuickTime will call us
    [(Controller *)gControllerObject resetPlayButtonForMovieStopState:nil];
}

pascal void MyCallBackProc (QTCallBack cb, long refcon)
{	//  This is our movie timebase callback routine.
    adjustPlayButtonForMyMovieControllerObject();
    [(Controller *)gControllerObject restoreMoviePlayCompleteCallBack];
}

void saveObjectForCallback(void *theObject)
{	// This routine stores a reference to our Controller object.
   gControllerObject = theObject;
}


